package org.hl7.fhir.utilities.xml;

/*
  Copyright (c) 2011+, HL7, Inc.
  All rights reserved.
  
  Redistribution and use in source and binary forms, with or without modification, 
  are permitted provided that the following conditions are met:
    
   * Redistributions of source code must retain the above copyright notice, this 
     list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright notice, 
     this list of conditions and the following disclaimer in the documentation 
     and/or other materials provided with the distribution.
   * Neither the name of HL7 nor the names of its contributors may be used to 
     endorse or promote products derived from this software without specific 
     prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
  IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
  NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
  POSSIBILITY OF SUCH DAMAGE.
  
 */



import java.io.IOException;
import java.util.Vector;

public class XMLWriterStateStack {

	private Vector<XMLWriterState> items = new Vector<XMLWriterState>();

	public int size(){
		return items.size();
	}
	
	public boolean empty(){
		return items.size() == 0;
	}
	
	public XMLWriterState current() throws IOException {
		if (empty())
			throw new IOException("stack is empty trying to get current");
		return items.get(items.size() - 1);
	}
	
	
	public void push(XMLWriterState element) {
		items.add(element);		
	}
		
	public void clear () {
		if (items != null)
		  items.clear();
	}

	public void pop() throws IOException {
		if (empty())
			throw new IOException("stack is empty trying to pop");
		if (current().isInComment())
			throw new IOException("Must close a comment sequence in the element in which it was started");
		
		items.remove(items.size() - 1);		
	}

	public XMLWriterState item(int index) {
		return items.get(index);
	}

	public boolean inComment() {
		for (int i = 0; i < items.size(); i++) {
			if (items.get(i).isInComment())
				return true;					
		}
		return false;
	}

}